import { world, system, BlockPermutation, ItemStack } from '@minecraft/server';

const processedBlocks = new Map();
const blockStates = new Map();

const DOOR_TYPES = [
    { name: "fir", loot: "fir_door" },
    { name: "jacaranda", loot: "jacaranda_door" },
    { name: "redwood", loot: "redwood_door" },
    { name: "baobab", loot: "baobab_door" },
    { name: "maple", loot: "maple_door" },
    { name: "forsaken_oak", loot: "forsaken_oak_door" },
    { name: "chorus", loot: "chorus_door" }
];

// Register all door logic for each type
DOOR_TYPES.forEach(({ name, loot }) => {
    const doorId = `biomes_and_beyond:${name}_door`;
    const onPlace = `biomes_and_beyond:${name}_on_place`;
    const onInteract = `biomes_and_beyond:${name}_on_interact`;
    const onTick = `biomes_and_beyond:${name}_on_tick`;

    // Register on_place
    world.beforeEvents.worldInitialize.subscribe(eventData => {
        eventData.blockComponentRegistry.registerCustomComponent(onPlace, {
            onPlace(e) {
                const { block } = e;
                const aboveBlock = block.above();
                const northBlock = block.north();
                const southBlock = block.south();
                const eastBlock = block.east();
                const westBlock = block.west();
                const { x, y, z } = block.location;

                if (processedBlocks.has(`${x},${y},${z}`)) return;
                const currentStates = block.permutation.getAllStates();
                const cardinalDirection = currentStates['minecraft:cardinal_direction'];

                if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('biomes_and_beyond:upper_block_bit')) {
                    const aboveBlockPermutation = BlockPermutation.resolve(doorId, {
                        'biomes_and_beyond:upper_block_bit': true,
                        'minecraft:cardinal_direction': cardinalDirection
                    });
                    aboveBlock.setPermutation(aboveBlockPermutation);
                    block.setPermutation(BlockPermutation.resolve(block.typeId, currentStates));
                }

                // Double door hinge logic (simplified, works for all directions)
                const directions = [
                    { dir: 'south', neighbor: eastBlock, aboveNeighbor: aboveBlock.east() },
                    { dir: 'east', neighbor: northBlock, aboveNeighbor: aboveBlock.north() },
                    { dir: 'north', neighbor: westBlock, aboveNeighbor: aboveBlock.west() },
                    { dir: 'west', neighbor: southBlock, aboveNeighbor: aboveBlock.south() }
                ];
                for (const { dir, neighbor, aboveNeighbor } of directions) {
                    if (cardinalDirection === dir && neighbor?.typeId === doorId) {
                        const neighborStates = neighbor.permutation.getAllStates();
                        if (!neighborStates['biomes_and_beyond:door_hinge_bit']) {
                            block.setPermutation(BlockPermutation.resolve(block.typeId, {
                                ...currentStates,
                                'biomes_and_beyond:door_hinge_bit': true
                            }));
                        }
                        const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                        if (aboveBlockCardinalDirection === dir && aboveNeighbor?.typeId === doorId) {
                            const aboveNeighborStates = aboveNeighbor.permutation.getAllStates();
                            if (!aboveNeighborStates['biomes_and_beyond:door_hinge_bit']) {
                                aboveBlock.setPermutation(BlockPermutation.resolve(aboveBlock.typeId, {
                                    ...aboveNeighborStates,
                                    'biomes_and_beyond:door_hinge_bit': true
                                }));
                            }
                        }
                    }
                }
                processedBlocks.set(`${x},${y},${z}`, true);
            }
        });
    });

    // Register on_interact
    world.beforeEvents.worldInitialize.subscribe(eventData => {
        eventData.blockComponentRegistry.registerCustomComponent(onInteract, {
            onPlayerInteract(e) {
                const { block, player } = e;
                const isUpperBlock = block.permutation.getState('biomes_and_beyond:upper_block_bit');
                const equipment = player.getComponent('equippable');
                const selectedItem = equipment.getEquipment('Mainhand');
                const targetBlock = isUpperBlock ? block.below() : block.above();
                const currentOpenState = block.permutation.getState('biomes_and_beyond:open_bit');
                const newOpenState = !currentOpenState;

                block.setPermutation(block.permutation.withState('biomes_and_beyond:open_bit', newOpenState));
                targetBlock.setPermutation(targetBlock.permutation.withState('biomes_and_beyond:open_bit', newOpenState));
                player.playSound(currentOpenState ? 'close.wooden_door' : 'open.wooden_door');

                // Water bucket logic (optional, as in your code)
                if (selectedItem?.typeId === 'minecraft:water_bucket') {
                    player.playSound('bucket.empty_water');
                    if (player.getGameMode() !== "creative") {
                        equipment.setEquipment('Mainhand', new ItemStack('minecraft:bucket', 1));
                    }
                    // Place structure and reapply states if needed (as in your code)
                }
            }
        });
    });

    // Register on_tick (redstone logic)
    world.beforeEvents.worldInitialize.subscribe(eventData => {
        eventData.blockComponentRegistry.registerCustomComponent(onTick, {
            onTick(e) {
                const { block } = e;
                const { x, y, z } = block.location;
                const blockKey = `${x},${y},${z}`;
                const currentState = block.permutation.getState('biomes_and_beyond:open_bit');
                const sound = currentState ? 'open.wooden_trapdoor' : 'close.wooden_trapdoor';

                const adjacentBlocks = {
                    north: block.north(),
                    east: block.east(),
                    south: block.south(),
                    west: block.west(),
                    above: block.above(),
                    below: block.below()
                };

                const hasRedstonePower = Object.values(adjacentBlocks).some(adjacentBlock =>
                    adjacentBlock?.getRedstonePower && adjacentBlock.getRedstonePower() > 0
                );
                const previousState = blockStates.get(blockKey) || false;
                const shouldOpen = hasRedstonePower && !previousState;
                const shouldClose = !hasRedstonePower && previousState;

                if (shouldOpen || shouldClose) {
                    const newState = shouldOpen;
                    block.setPermutation(block.permutation.withState('biomes_and_beyond:open_bit', newState));
                    block.dimension.playSound(sound, block.location);
                    blockStates.set(blockKey, newState);

                    // Update the other half of the door
                    const isUpperBlock = block.permutation.getState('biomes_and_beyond:upper_block_bit');
                    const correspondingBlock = isUpperBlock ? adjacentBlocks.below : adjacentBlocks.above;
                    if (correspondingBlock?.typeId === doorId) {
                        correspondingBlock.setPermutation(correspondingBlock.permutation.withState('biomes_and_beyond:open_bit', newState));
                    }
                }
            }
        });
    });

    // Loot and block break logic
    system.run(() => {
        world.afterEvents.playerBreakBlock.subscribe(eventData => {
            const { block } = eventData;
            const { x, y, z } = block.location;
            const belowBlock = block.below();
            if (belowBlock.typeId === doorId) {
                world.getDimension('overworld').runCommand(`loot spawn ${x} ${y - 1} ${z} loot "blocks/${loot}"`);
                belowBlock.setType('minecraft:air');
            }
            processedBlocks.delete(`${x},${y},${z}`);
            processedBlocks.delete(`${x},${y-1},${z}`);
        });
    });

    // Prevent placing on top of another door
    if (world.beforeEvents && world.beforeEvents.playerPlaceBlock) {
        world.beforeEvents.playerPlaceBlock.subscribe(event => {
            const { block } = event;
            const belowBlock = block.below();
            if (belowBlock.typeId === doorId) {
                event.cancel = true;
            }
        });
    }
});