import { world, system } from "@minecraft/server";

const activePlayers = new Map(); // Tracks active knockback for players

const PuffshroomComponent = {
    onEntityFallOn(event) {
        const { entity, block } = event;

        if (block.permutation.matches("biomes_and_beyond:puffshroom")) {
            const growthState = block.permutation.getState("biomes_and_beyond:growth");

            if (growthState !== undefined) {
                const knockbackHeights = {
                    0: 0.45,
                    1: 0.66,
                    2: 0.85,
                    3: 1.25,
                };

                const slowFallingDurations = {
                    0: 1,
                    1: 1,
                    2: 2,
                    3: 3,
                };

                const launchPower = knockbackHeights[growthState] || 0.1;
                const slowFallingDuration = slowFallingDurations[growthState] || 2;

                if (entity?.typeId === "minecraft:player") {
                    // Track player interaction
                    activePlayers.set(entity.id, { launchPower, slowFallingDuration });

                    // Trigger particle effects at block's location
                    const { x, y, z } = block.location;
                    entity.runCommandAsync(
                        `particle minecraft:wind_explosion_emitter ${x} ${y} ${z}`
                    ).catch((error) => {
                        console.error(`Failed to spawn particle: ${error}`);
                    });

                    // Play sound at the player's position
                    const { x: px, y: py, z: pz } = entity.location;
                    entity.runCommandAsync(
                        `playsound wind_charge.burst @a ${px} ${py} ${pz} 0.3 1`
                    ).catch((error) => {
                        console.error(`Failed to play sound: ${error}`);
                    });
                } else {
                    // For mobs, apply impulse to simulate upward motion
                    try {
                        entity.clearVelocity(); // Clear existing velocity
                        entity.applyImpulse({ x: 0, y: launchPower, z: 0 }); // Apply upward motion

                        // Trigger particle effects at mob's location
                        const { x: mx, y: my, z: mz } = entity.location;
                        entity.runCommandAsync(
                            `particle minecraft:wind_explosion_emitter ${mx} ${my} ${mz}`
                        ).catch((error) => {
                            console.error(`Failed to spawn particle for mob: ${error}`);
                        });

                        // Play sound at mob's position
                        entity.runCommandAsync(
                            `playsound wind_charge.burst @a ${mx} ${my} ${mz} 0.3 1`
                        ).catch((error) => {
                            console.error(`Failed to play sound for mob: ${error}`);
                        });

                        // Apply Slow Falling effect for mobs
                        entity.runCommandAsync(
                            `effect @s slow_falling ${slowFallingDuration} 1 true`
                        ).catch((error) => {
                            console.error(`Failed to apply Slow Falling effect for mob: ${error}`);
                        });
                    } catch (error) {
                        console.error(`Error applying impulse to mob: ${error}`);
                    }
                }
            }
        }
    },
};

system.runInterval(() => {
    for (const player of world.getPlayers()) {
        if (activePlayers.has(player.id)) {
            const { launchPower, slowFallingDuration } = activePlayers.get(player.id);
            activePlayers.delete(player.id);

            // Apply knockback to the player
            player.applyKnockback(0, 0, 0, launchPower);

            // Apply Slow Falling effect with appropriate duration
            player.runCommandAsync(
                `effect @s slow_falling ${slowFallingDuration} 1 true`
            ).catch((error) => {
                console.error(`Failed to apply Slow Falling effect: ${error}`);
            });
        }
    }
}, 1);

world.beforeEvents.worldInitialize.subscribe(({ blockComponentRegistry }) => {
    blockComponentRegistry.registerCustomComponent(
        "biomes_and_beyond:puffshroom_component",
        PuffshroomComponent
    );
});