import { world, system } from "@minecraft/server";
import { ModalFormData, ActionFormData, MessageFormData } from "@minecraft/server-ui";

// Utility function to play the page turn sound at a player's location
function playPageTurnSound(player) {
    try {
        player.runCommandAsync('playsound item.book.page_turn');
    } catch (e) {
        // Ignore errors if sound fails
    }
}

function ensureScoreboard() {
    if (!world.scoreboard.getObjective("joined")) {
        world.scoreboard.addObjective("joined", "Joined Players");
        console.log("Scoreboard 'joined' initialized.");
    }
}

function ensureEffectTags(player) {
    if (!player) return;

    let changed = false;

    if (!player.hasTag("leaf_enabled")) {
        player.addTag("leaf_enabled");
        player.removeTag("leaf_disabled");
        changed = true;
    }

    if (!player.hasTag("storms_enabled")) {
        player.addTag("storms_enabled");
        player.removeTag("storms_disabled");
        changed = true;
    }

    if (!player.hasTag("end_fog_enabled")) {
        player.addTag("end_fog_enabled");
        player.removeTag("end_fog_disabled");
        changed = true;
    }
}

world.afterEvents.playerSpawn.subscribe(event => {
    const player = event?.player;
    if (!player) return;

    system.runTimeout(() => {
        ensureEffectTags(player);

        const joinedObjective = world.scoreboard.getObjective("joined");
        if (joinedObjective) {
            joinedObjective.setScore(player, 1);
        }
    }, 10);
});

function toggleSettings(player) {
    if (!player) return;
    
    const isStormEnabled = player.hasTag("storms_enabled");

    new ModalFormData()
        .title("Biomes & Beyond Global Settings")
        .toggle("Sandstorms & Blizzards", isStormEnabled)
        .show(player)
        .then(response => {
            if (response.canceled) return;
            playPageTurnSound(player);
            const toggleStorms = response.formValues[0];

            world.getPlayers().forEach(p => {
                if (toggleStorms) {
                    p.addTag("storms_enabled");
                    p.removeTag("storms_disabled");
                } else {
                    p.addTag("storms_disabled");
                    p.removeTag("storms_enabled");
                }
            });

            world.sendMessage(toggleStorms ? "§aStorm effects enabled." : "§cStorm effects disabled.");
        });
}

function togglePersonalSettings(player) {
    if (!player) return;

    const isLeafEnabled = player.hasTag("leaf_enabled");
    const isFogEnabled = player.hasTag("end_fog_enabled");

    new ModalFormData()
        .title("Biomes & Beyond Personal Settings")
        .toggle("Ambient Particles §7§o(custom only)§r", isLeafEnabled)
        .toggle("End Biome Fog", isFogEnabled)
        .show(player)
        .then(response => {
            if (response.canceled) return;
            playPageTurnSound(player);
            const toggleLeaves = response.formValues[0];
            const toggleFog = response.formValues[1];

            if (toggleLeaves) {
                player.addTag("leaf_enabled");
                player.removeTag("leaf_disabled");
            } else {
                player.addTag("leaf_disabled");
                player.removeTag("leaf_enabled");
            }

            if (toggleFog) {
                player.addTag("end_fog_enabled");
                player.removeTag("end_fog_disabled");
            } else {
                player.addTag("end_fog_disabled");
                player.removeTag("end_fog_enabled");
            }
        });
}

world.afterEvents.itemUse.subscribe(event => {
    const player = event?.source;
    if (!player) return;

    if (event.itemStack?.typeId === "biomes_and_beyond:global_settings") toggleSettings(player);
    if (event.itemStack?.typeId === "biomes_and_beyond:personal_settings") togglePersonalSettings(player);
});


// Function to show the wiki main menu
function showWikiForm(player) {
    const form = new ActionFormData()
        .title("B&B Quick Wiki (Incomplete)")
        .button("Mobs", "textures/ui/wiki/mob_icon.png")
        .button("Items", "textures/ui/icon_recipe_item.png")
        .button("Tools & Equipment", "textures/ui/icon_recipe_equipment.png")
        .button("Biomes", "textures/ui/world_glyph_color.png")
        .button("Structures", "textures/ui/wiki/structure_icon.png")
        .button("Ores", "textures/ui/icon_recipe_nature.png")
        .button("More", "textures/ui/more-dots.png");

    form.show(player).then(response => {
        if (!response.canceled) {
            playPageTurnSound(player);
            switch (response.selection) {
                case 0:
                    showMobsList(player);
                    break;
                case 1:
                    showItemsList(player);
                    break;
                case 2:
                    showToolList(player);
                    break;
                case 3:
                    showBiomesList(player);
                    break;
                case 4:
                    showStructureList(player);
                    break;
                case 5:
                    showOreList(player);
                    break;    
                case 6:
                    showOtherCategoriesList(player);
                    break;
            }
        }
    });
}

// Function to show a list of biomes
function showBiomesList(player) {
    const form = new ActionFormData()
        .title("Biomes")
        .button("Aspen Forest")
        .button("Lavender Fields")
        .button("Autumn Woods")
        .button("Badlands")
        .button("Prairie")
        .button("Savanna")
        .button("Deciduous Forest")
        .button("Pendula Birch Forest")
        .button("Boggy Swamplands")
        .button("Pale Garden")
        .button("Forsaken Depths")
        .button("Chorus Forest")
        .button("Velvium Expanses")
        .button("Depraved Wetlands");

    form.show(player).then(response => {
        if (!response.canceled) {
            playPageTurnSound(player);
            const biomeDescriptions = {
                0: ["Aspen Forest", "A forest filled with tall yellow Aspen trees, wildflowers, and shrubs.\n\n\n"],
                1: ["Lavender Fields", "A flat serene biome filled with purple Jacaranda trees, Wisteria, and Lavender flowers.\n\n\n"],
                2: ["Autumn Woods", "Fall-like biome filled with red and yellow Maple trees, orange oak trees, and pumpkins.\n\n\n"],
                3: ["Badlands", "Hot and dry biome with red rock formations, cacti, and sparse vegetation, with dry oak trees scattering the landscape.\n\n\n"],
                4: ["Prairie", "Mostly flat biome with tall grass, Clovers, and a few oak trees.\n\n\n"],
                5: ["Savanna", "Hot and dry biome with Strata, tall grass, acacia, and Baobab trees.\n\n\n"],
                6: ["Deciduous Forest", "A dense forest filled with oak trees of various sizes, flowers, and other vegetation.\n\n\n"],
                7: ["Pendula Birch Forest", "A forest filled with tall white birch trees, bees, Daisy Petals, and bushes.\n\n\n"],
                8: ["Boggy Swamplands", "A wet and moist biome with dark water, tall grass, and various plants.\n\n\n"],
                9: ["Pale Garden", "The pale garden has recieved a slight makeover, allowing the trees to be taller, added pale bushes, and added a dense pale fog.\n\n\n"],
                10: ["Forsaken Depths", "A dark and eerie biome with fossils, forsaken oak trees, hanging lichen, lichen carpet, and whispering willows.\n\n\n"],
                11: ["Chorus Forest", "An end biome filled with chorus trees, Void Blossoms, Void Blossoms Bushes, and a unique lively, but still lonely atmosphere.\n\n\n"],
                12: ["Velvium Expanses", "A vibrant and colorful biome with Velvium trees, Velvium Vines, and Cobbled End Stone boulders.\n\n\n"],
                13: ["Depraved Wetlands", "Wet, swamp-like biome with giant mushroom trees and depraved vegetation.\n\n\n"],
            };

            if (biomeDescriptions[response.selection]) {
                const [name, description] = biomeDescriptions[response.selection];
                showBiomeInfo(player, name, description);
            }
        }
    });
}

// Function to show biome details
function showBiomeInfo(player, name, description) {
    const form = new MessageFormData()
        .title(name)
        .body(description)
        .button1("Back to Biomes")
        .button2("Close");

    form.show(player).then(response => {
        if (response.selection === 0) {
            playPageTurnSound(player);
            showBiomesList(player);
        }
    });
}

// Function to show a list of mobs
function showMobsList(player) {
    const form = new ActionFormData()
        .title("Mobs")
        .button("Deer", "textures/ui/wiki/deer.png")
        .button("Grizzly Bear", "textures/ui/wiki/bear_brown.png")
        .button("Raccoon", "textures/ui/wiki/raccoon.png")
        .button("Crocodile", "textures/ui/wiki/crocodile.png")
        .button("Mummy", "textures/ui/wiki/mummy.png")
        .button("Warthog", "textures/ui/wiki/warthog.png")
        .button("Dweller", "textures/ui/wiki/dweller.png")
        .button("Lurker", "textures/ui/wiki/lurker.png")
        .button("Spectre", "textures/ui/wiki/spectre.png")
        .button("Voidcaller", "textures/ui/wiki/voidcaller.png")
        .button("Sigiling", "textures/ui/wiki/sigiling.png")
        .button("Hollowed", "textures/ui/wiki/hollowed.png")
        .button("Puffling", "textures/ui/wiki/puffling.png")
        .button("Chimp", "textures/ui/wiki/chimp.png");

    form.show(player).then(response => {
        if (!response.canceled) {
            playPageTurnSound(player);
            const mobDescriptions = {
                0: ["Deer", "Passive mobs that will avoid players, villagers, and pillagers. They will drop venison, deer antlers, leather and a rare chance to drop a deer trophy when killed.\n\nThey can be fed carrots to breed. You can also find the rare white deer and have an even rarer chance to obtain the SUPER rare white deer trophy...phew, so many rares!\n\n\n"],
                1: ["Grizzly Bear", "Neutral mob that will attack players if provoked or if a player comes too close. Can be bred using honeycomb, but it's dangerous.\n\nDrops grizzly pelt when killed.\n\n\n"],
                2: ["Raccoon", "Adorable neutral mobs that will attack players if angered. They will avoid players, and are nocturnal and will sleep during the day similar to foxes.\n\nThey will eat crops, and pickup most food items and eat them. They are afraid of carved pumpkins, so they are a good way to prevent raccoons from eating crops.\n\nTo tame raccoons, sneak and feed them bread.\n\n\n"],
                3: ["Crocodile", "Spawns primarily in water, where it is fastest, and will attack most mobs.\n\nDrops Crocodile meat when killed. Cannot be bred.\n\n\n"],
                4: ["Mummy", "An ancient corpse from a past civilization wrapped in dirty cloth.\n\nDrops paper, gold and rotten flesh when killed.\n\n\n"],
                5: ["Warthog", "Warthogs are passive creatures found in the Savanna biome, attacking players only when provoked.\n\nPlayers can saddle up a Warthog and ride it using a carrot on a stick, much like riding pigs. Warthogs occasionally dig through nearby blocks in search of food, sometimes unearthing other items in the process.\n\nWhen they find food, they will eat it. These creatures closely resemble the Nether's Hoglins and will transform into Hoglins if they stay in the Nether too long.\n\n\n"],
                6: ["Dweller", "Dwellers are mysterious cave-dwelling monsters found within the Forsaken Depths.\n\nThey inhabit deep caves, spawning between levels -63 and 8. Although their eyesight is poor, they compensate with a keen sense of smell and acute hearing. Periodically, they will seek out players within a 24-block radius and attack once within 6 blocks.\n\nIf the player runs outside this range without being hit, the Dwellers temporarily lose sight of them. Dwellers have a strong hatred to fire, making torches, standing torches, campfires, and lanterns effective repellents.\n\nWhile they can destroy torches, they are unable to break standing torches, campfires, and lanterns. Their origin is unknown, but they are believed to be long-lost adventurers who couldn't find their way out of the deep cave systems. Over time, they were slowly overcome by the darkness, leading to mutation and a loss of all sense of who they once were.\n\n\n"],
                7: ["Lurker", "Lurkers are a newly introduced hostile mob found deep within the Forsaken Depths.\n\nThey're known for their haunting cries, which can be heard from far away. Generally, they try to avoid conflict but become hostile if a player gets too close. When attacked, they blind the player and then flee, continuing to cry.\n\nSimilar to Dwellers, the darkness has severely impaired their vision, so they rely heavily on sound and smell. It's believed that they cry because they have faint memories of their former selves and dislike having players nearby.\n\n\n"],
                8: ["Spectre", "Spectres are hostile mobs in the Forsaken Depths.\n\nThey are surrounded by a thick black mist and have bright white eyes that shine through the darkness. They will vanish and sometimes even attack if looked at or if a player gets too close.\n\nThese ghostly beings are thought to be lost souls corrupted and trapped in the Forsaken Depths.\n\n\n"],
                9: ["Voidcaller", "Voidcallers are an Enderman variant that watch players from afar, waiting for a player to gaze at the bright yellow sigil on its chest, or at its eyes.\n\nOnce a player angers a Voidcaller, they will begin to shriek, angering surrounding enderlings who will also begin to attack the player.\n\n\n"],
                10: ["Sigiling", "Sigilings are another variant of Endermen, these hunky endermen will shoot projectiles from its mouth when angered.\n\nThey will attempt to avoid the player's attacks as much as possible, while remaining offensive. They are significantly slower than other enderlings, but make up for it in damage and health.\n\n\n"],
                11: ["Hollowed", "Hollowed spawn in the Depraved Wetlands and become hostile if the player gets too close, even when wearing a carved pumpkin. They are noticeably weaker and slower than regular Endermen.\n\nWhen killed, a Hollowed splits into two, with each duplicate having only about half of it's orignal health.\n\nTheir teleportation ability is not great, allowing them to teleport only up to 7 blocks horizontally.\n\n\n"],
                12: ["Puffling", "Pufflings are passive creatures native to the Velvium Expanses.\n\nThey tend to avoid players unless approached while crouching. To breed Pufflings, players must feed them wind charges while crouched. Once a baby Puffling is born, it becomes friendly toward the player and can perch on their shoulders, occasionally granting them bursts of slow falling.\n\nHowever, if provoked, Pufflings will become hostile and attack by spitting wind charges at their target. Pufflings will attack skeletons and zombies on sight.\n\n\n"],
                13: ["Chimp", "Chimps can be found in Jungles, they are neutral animals and will only attack when provoked. They will pickup and eat dropped food and will even pickup majority of items. When picking up any tools and armor, they will equip and use them, so be careful not to lose your gear near them because they will not give it back willingly. They can be fed apples to breed.\n\n\n"]
            };

            if (mobDescriptions[response.selection]) {
                const [name, description] = mobDescriptions[response.selection];
                showMobInfo(player, name, description);
            }
        }
    });
}

// Function to show mob details
function showMobInfo(player, name, description) {
    const form = new MessageFormData()
        .title(name)
        .body(description)
        .button1("Back to Mobs")
        .button2("Close");

    form.show(player).then(response => {
        if (response.selection === 0) {
            playPageTurnSound(player);
            showMobsList(player);
        }
    });
}

// Function to show a list of items
function showItemsList(player) {
    const form = new ActionFormData()
        .title("Items")
        .button("Deer Antlers", "textures/items/deer_antler.png")
        .button("Deer Hide", "textures/items/deer_hide.png")
        .button("Dweller Flesh", "textures/items/dweller_flesh.png")
        .button("Aloe Vera Leaf", "textures/items/aloe.png")
        .button("Coconut", "textures/items/coconut.png")
        .button("Grizzly Bear Pelt", "textures/items/grizzly_pelt.png")
        .button("Lurker's Foot", "textures/items/lurker_foot.png")
        .button("Sulfur", "textures/items/sulfur.png")
        .button("Raw Tungsten", "textures/items/tungsten_raw.png")
        .button("Tungsten Ingot", "textures/items/tungsten_ingot.png");

    form.show(player).then(response => {
        if (!response.canceled) {
            playPageTurnSound(player);
            const itemDescriptions = {
                0: ["Deer Antlers", "Dropped by deer, can be used to craft bonemeal."],
                1: ["Deer Hide", "Dropped by deer. (Will be tradeable in a future update.)"],
                2: ["Dweller Flesh", "Dropped by Dwellers, can be used in a funrace to get leather."],
                3: ["Aloe Vera Leaf", "Dropped by mining an Aloe Vera plant. Can be eaten."],
                4: ["Coconut",  "Dropped by mining coconuts found on palm trees. Can be split in any crafting grid to eat, or be thrown.\n\nCoconuts can be used to craft Coconut Palm Saplings, which can then be planted to grow more Coconut Palms."],
                5: ["Grizzly Bear Pelt", "Dropped by grizzly bears. (Will be tradeable in a future update.)"],
                6: ["Lurker's Foot", "Dropped rarely by lurkers. (Will be useable in a future update.)"],
                7: ["Sulfur", "Dropped by mining Sulfur Ore. Can be used to craft gunpowder, used as a fuel source, or crafted into a bag of sulfur."],
                8: ["Raw Tungsten", "Dropped by mining Tungsten Ore. Can be smelted in a furnace to obtain Tungsten Ingots."],
                9: ["Tungsten Ingot", "Obtained from smelting raw Tungsten in a furnace."]
            };
            if (itemDescriptions[response.selection]) {
                const [name, description] = itemDescriptions[response.selection];
                showItemInfo(player, name, description);
            }
        }
    });
}

// Function to show item details
function showItemInfo(player, name, description) {
    const form = new MessageFormData()
        .title(name)
        .body(description)
        .button1("Back to Items")
        .button2("Close");

    form.show(player).then(response => {
        if (response.selection === 0) {
            playPageTurnSound(player);
            showItemsList(player);
        }
    });
}

// Function to show a list of other categories
function showOtherCategoriesList(player) {
    const form = new ActionFormData()
        .title("Other Categories")
        .button("Credits", "textures/ui/Feedback.png"); 

    form.show(player).then(response => {
        if (!response.canceled) {
            playPageTurnSound(player);
            switch (response.selection) {
                case 0:
                    showCredits(player);
                    break;
            }
        }
    });
}


function showCredits(player) {
    const playerName = player.name || "Player";

    const form = new MessageFormData()
        .title("Credits")
        .body(`Created & developed by iKorbon.\n\n\nSpecial thanks to:\nCooperCan\ngeertop\nPoggy\nKiaoga\nAnd you, ${playerName}. Thank you for playing, and have fun!\n\n`)
        .button1("Back")
        .button2("Close");

    form.show(player).then(response => {
        if (response.selection === 0) {
            playPageTurnSound(player);
            showOtherCategoriesList(player);
        }
    });
}

// Function to show a list of tools
function showToolList(player) {
    const form = new ActionFormData()
        .title("Tools & Equipment")
        .button("Khopesh", "textures/items/khopesh.png")
        .button("Tungsten Tools & Equipment", "textures/items/tungsten_helmet.png");

    form.show(player).then(response => {
        if (!response.canceled) {
            playPageTurnSound(player);
            const toolDescriptions = {
                0: ["Khopesh", "An ancient sword left behind by a past civilization.\n\n-Deals 6 damage\n-Has 350 durability\n-Has a chance to inflict instant damage\n-Dropped rarely by Mummies\n\n"],
                1: ["Tungsten Tools & Equipment", "-Slightly more durability compared to diamond tools ( 1758 )\n-Slightly less enchantable compared to diamond tools (9)\n-Slightly slower mining speed compared to diamond tools\n-Same amount of attack damage as diamond tools\n\n"]
            };

            if (toolDescriptions[response.selection]) {
                const [name, description] = toolDescriptions[response.selection];
                showToolInfo(player, name, description);
            }
        }
    });
}

// Function to show tool details
function showToolInfo(player, name, description) {
    const form = new MessageFormData()
        .title(name)
        .body(description)
        .button1("Back to Tools & Equipment")
        .button2("Close");

    form.show(player).then(response => {
        if (response.selection === 0) {
            playPageTurnSound(player);
            showToolList(player);
        }
    });
}

// Function to show a list of biomes
function showStructureList(player) {
    const form = new ActionFormData()
        .title("Structures")
        .button("Trial Tower")
        .button("Goblin Hut")
        .button("Ruined Altars");

    form.show(player).then(response => {
        if (!response.canceled) {
            playPageTurnSound(player);
            const structureDescriptions = {
                0: ["Trail Tower", "A tall tower containing several levels of trial spawners to fight through to obtain the loot at the top.\n\nSpawns in: Forests, Pendula Birch Forests, Mystic Roofed Forest, Prairie Plains, Desert, Mesa, Coniferous Forest\n\n\n"],
                1: ["Goblin Hut", "Homes of Goblins, spawning up to several at a time.\n\nSpawns within the Mystic Roofed Forest.\n\n\n"],
                2: ["Ruined Altars", "Old, abandonded altars that served an unknown purpose. Now dilapidated, and forgotten due to age.\n\nSpawns in: Forests, Taigas, and Mystic Roofed Forests.\n\n\n"],
            };

            if (structureDescriptions[response.selection]) {
                const [name, description] = structureDescriptions[response.selection];
                showStructureInfo(player, name, description);
            }
        }
    });
}

// Function to show structure details
function showStructureInfo(player, name, description) {
    const form = new MessageFormData()
        .title(name)
        .body(description)
        .button1("Back to Structures")
        .button2("Close");

    form.show(player).then(response => {
        if (response.selection === 0) {
            playPageTurnSound(player);
            showStructureList(player);
        }
    });
}

// Function to show a list of ores
function showOreList(player) {
    const form = new ActionFormData()
        .title("Ores")
        .button("Sulfur Ore", "textures/ui/wiki/sulfur_ore.png")
        .button("Tungsten Ore", "textures/ui/wiki/tungsten_ore.png");

    form.show(player).then(response => {
        if (!response.canceled) {
            playPageTurnSound(player);
            const oreDescriptions = {
                0: ["Sulfur Ore", "A common ore found in the Overworld. It can be smelted into Sulfur, which is used for crafting gunpowder or as a fuel source."],
                1: ["Tungsten Ore", "A rare ore found anywhere in the Overworld and in The End with a triangular distribution, between levels 16 through -64 in the overworld, and on the lower levels of end islands.\n\nSmelting produces Tungsten Ingots, which are used for crafting durable tools and armor."]
            };

            if (oreDescriptions[response.selection]) {
                const [name, description] = oreDescriptions[response.selection];
                showOreInfo(player, name, description);
            }
        }
    });
}

// Function to show ore details
function showOreInfo(player, name, description) {
    const form = new MessageFormData()
        .title(name)
        .body(description)
        .button1("Back to Ores")
        .button2("Close");

    form.show(player).then(response => {
        if (response.selection === 0) {
            playPageTurnSound(player);
            showOreList(player);
        }
    });
}

// Ensure the itemUse event subscription is handled correctly as per the original code
world.afterEvents.itemUse.subscribe(event => {
    const player = event.source;
    const item = event.itemStack;

    if (player && item && item.typeId === "biomes_and_beyond:info_book") {
        showWikiForm(player);
    }
});