import { world } from "@minecraft/server";

const validEffectTypes = new Set([
  "absorption", "bad_omen", "blindness", "clear", "conduit_power", "darkness",
  "fatal_poison", "fire_resistance", "haste", "health_boost", "hunger", 
  "instant_damage", "instant_health", "invisibility", "jump_boost", 
  "levitation", "mining_fatigue", "nausea", "night_vision", "poison", 
  "regeneration", "resistance", "saturation", "slow_falling", "slowness", 
  "speed", "strength", "village_hero", "water_breathing", "weakness", "wither"
]);

const toolEffects = [
  {
    toolTypeId: "biomes_and_beyond:gold_khopesh",
    effects: [
      { 
        effectType: "instant_damage", 
        duration: 1, 
        amplifier: 0, 
        showParticles: true, 
        target: "attacked", 
        chance: 50,
        targetType: "mob",  // This effect applies to mobs only
        excludeMonsters: ["minecraft:zombie", "minecraft:skeleton"], // Exclude specific monsters
        familyTags: ["monster"] // Effect applies only to entities in the "monster" family tag
      },
    ]
  }
];

world.afterEvents.entityHitEntity.subscribe(evd => {
  const { damagingEntity, hitEntity } = evd;

  // Ignore if damaging entity is not a player
  if (damagingEntity.typeId !== 'minecraft:player') return;

  const playerEquippableComp = damagingEntity.getComponent("equippable");
  if (!playerEquippableComp) return;

  const heldItem = playerEquippableComp.getEquipment("Mainhand");
  if (!heldItem) return;

  const foundTool = toolEffects.find(tool => tool.toolTypeId === heldItem.typeId);
  if (foundTool) {
    foundTool.effects.forEach(effectInfo => {
      const { effectType, duration, amplifier, showParticles, target, chance, targetType, excludeMonsters, familyTags } = effectInfo;

      if (!validEffectTypes.has(effectType)) {
        return console.error(`§cError:§r §7Invalid effect type '§r${effectType}§7' detected.`);
      }

      const durationInTicks = 20 * duration;
      if (durationInTicks < 1 || durationInTicks > 20000000) {
        return console.error(`§cError:§r §7Duration value '§r${duration}§7' is not within the valid range (0 to 1,000,000 seconds).`);
      }

      if (amplifier < 0 || amplifier > 255) {
        return console.error(`§cError:§r §7Amplifier value '§r${amplifier}§7' is not within the valid range (0 to 255).`);
      }

      if (typeof showParticles !== 'boolean') {
        return console.error(`§cError:§r §7Invalid value '§r${showParticles}§7' for showParticles. It must be a boolean type (true or false).`);
      }

      const randomChance = Math.floor(Math.random() * 100) + 1;
      if (randomChance <= chance) {
        let targetEntity = null;

        // Ensure familyTags are strings
        const isFamilyTagExcluded = familyTags && familyTags.some(tag => typeof tag === "string" && hitEntity.hasTag(tag));

        // Check if hitEntity is in any of the excluded monster list
        const isExcludedMonster = excludeMonsters.includes(hitEntity.typeId);

        // Apply the effect if the entity is not excluded
        if (!isExcludedMonster && !isFamilyTagExcluded) {
          // Apply effect based on targetType
          if (targetType === "player") {
            if (hitEntity.typeId === 'minecraft:player') {
              targetEntity = hitEntity;  // Apply effect to the player
            }
          } else if (targetType === "mob") {
            if (hitEntity.typeId !== 'minecraft:player') {
              targetEntity = hitEntity;  // Apply effect to the mob
            }
          } else if (targetType === "all") {
            targetEntity = hitEntity;  // Apply effect to both players and mobs
          }

          // If a valid targetEntity is found, apply the effect
          if (targetEntity) {
            const targetToApply = target === "attacker" ? damagingEntity : targetEntity;
            targetToApply.addEffect(effectType, durationInTicks, { amplifier, showParticles });
          }
        }
      }
    });
  }
});
